@@ -17,14 +17,14 @@
 	if (rc) {
 		int fd = fileno(rc);
 
-		pseudo_debug(2, "fopen64 '%s': fd %d <FILE %p>\n", path, fd, (void *) rc);
+		pseudo_debug(PDBGF_FILE, "fopen64 '%s': fd %d <FILE %p>\n", path, fd, (void *) rc);
 		if (real___fxstat64(_STAT_VER, fd, &buf) != -1) {
 			if (!existed) {
 				pseudo_client_op(OP_CREAT, 0, -1, -1, path, &buf);
 			}
 			pseudo_client_op(OP_OPEN, pseudo_access_fopen(mode), fd, -1, path, &buf);
 		} else {
-			pseudo_debug(1, "fopen64 (fd %d) succeeded, but fstat failed (%s).\n",
+			pseudo_debug(PDBGF_CONSISTENCY, "fopen64 (fd %d) succeeded, but fstat failed (%s).\n",
 				fd, strerror(errno));
 			pseudo_client_op(OP_OPEN, pseudo_access_fopen(mode), fd, -1, path, 0);
 		}
