@@ -16,14 +16,14 @@
 	if (rc) {
 		int fd = fileno(rc);
 
-		pseudo_debug(2, "fopen '%s': fd %d <FILE %p>\n", path, fd, (void *) rc);
+		pseudo_debug(PDBGF_OP, "fopen '%s': fd %d <FILE %p>\n", path, fd, (void *) rc);
 		if (base_fstat(fd, &buf) != -1) {
 			if (!existed) {
 				pseudo_client_op(OP_CREAT, 0, -1, -1, path, &buf);
 			}
 			pseudo_client_op(OP_OPEN, pseudo_access_fopen(mode), fd, -1, path, &buf);
 		} else {
-			pseudo_debug(1, "fopen (fd %d) succeeded, but fstat failed (%s).\n",
+			pseudo_debug(PDBGF_CONSISTENCY, "fopen (fd %d) succeeded, but fstat failed (%s).\n",
 				fd, strerror(errno));
 			pseudo_client_op(OP_OPEN, pseudo_access_fopen(mode), fd, -1, path, 0);
 		}
