@@ -75,6 +75,8 @@ int pseudo_umask = 022;
 
 static char **fd_paths = NULL;
 static int nfds = 0;
+static const char **passwd_paths = NULL;
+static int npasswd_paths = 0;
 static int messages = 0;
 static struct timeval message_time = { .tv_sec = 0 };
 static int pseudo_inited = 0;
@@ -93,7 +95,7 @@ gid_t pseudo_egid;
 gid_t pseudo_sgid;
 gid_t pseudo_fgid;
 
-#define PSEUDO_ETC_FILE(filename, realname, flags) pseudo_etc_file(filename, realname, flags, (char *[]) { pseudo_chroot, pseudo_passwd, PSEUDO_PASSWD_FALLBACK }, PSEUDO_PASSWD_FALLBACK ? 3 : 2)
+#define PSEUDO_ETC_FILE(filename, realname, flags) pseudo_etc_file(filename, realname, flags, passwd_paths, npasswd_paths)
 
 /* helper function to make a directory, just like mkdir -p.
  * Can't use system() because the child shell would end up trying
@@ -117,6 +119,73 @@ mkdir_p(char *path) {
 	(void) mkdir(path, 0755);
 }
 
+/* Populating an array of unknown size is one of my least favorite
+ * things. The idea here is to ensure that the logic flow is the same
+ * both when counting expected items, and when populating them.
+ */
+static void
+build_passwd_paths(void)
+{
+	int np = 0;
+	int pass = 0;
+	
+	/* should never happen... */
+	if (passwd_paths) {
+		free(passwd_paths);
+		passwd_paths = 0;
+		npasswd_paths = 0;
+	}
+
+#define SHOW_PATH pseudo_debug(PDBGF_CHROOT | PDBGF_VERBOSE, "passwd_paths[%d]: '%s'\n", np, (passwd_paths[np]))
+#define ADD_PATH(p) do { if (passwd_paths) { passwd_paths[np] = (p); SHOW_PATH; } ++np; } while(0)
+#define NUL_BYTE(p) do { if (passwd_paths) { *(p)++ = '\0'; } else { ++(p); } } while(0)
+
+	do {
+		if (pseudo_chroot) {
+			ADD_PATH(pseudo_chroot);
+		}
+		if (pseudo_passwd) {
+			char *s = pseudo_passwd;
+			while (s) {
+				char *t = strchr(s, ':');
+				if (t) {
+					NUL_BYTE(t);
+				}
+				ADD_PATH(s);
+				s = t;
+			}
+		}
+		if (PSEUDO_PASSWD_FALLBACK) {
+			ADD_PATH(PSEUDO_PASSWD_FALLBACK);
+		}
+
+		/* allocation and/or return */
+		if (passwd_paths) {
+			if (np != npasswd_paths) {
+				pseudo_diag("internal error: path allocation was inconsistent.\n");
+			} else {
+				/* yes, we allocated one extra for a trailing
+				 * null pointer.
+				 */
+				passwd_paths[np] = NULL;
+			}
+			return;
+		} else {
+			passwd_paths = malloc((np + 1) * sizeof(*passwd_paths));
+			npasswd_paths = np;
+			if (!passwd_paths) {
+				pseudo_diag("couldn't allocate storage for password paths.\n");
+				exit(1);
+			}
+			np = 0;
+		}
+	} while (++pass < 2);
+	/* in theory the second pass already returned, but. */
+	pseudo_diag("should totally not have gotten here.\n");
+
+	return;
+}
+
 void
 pseudo_init_client(void) {
 	char *env;
@@ -325,9 +394,12 @@ pseudo_init_client(void) {
 
 		env = pseudo_get_value("PSEUDO_PASSWD");
 		if (env) {
+			/* note: this means that pseudo_passwd is a
+			 * string we're allowed to modify... */
 			pseudo_passwd = strdup(env);
 		}
 		free(env);
+		build_passwd_paths();
 
 		pseudo_inited = 1;
 	}
@@ -408,6 +480,7 @@ pseudo_file_open(char *name, int *fd, FILE **fp) {
  */
 int
 pseudo_pwd_lck_open(void) {
+	pseudo_pwd_lck_close();
 	if (!pseudo_pwd_lck_name) {
 		pseudo_pwd_lck_name = malloc(pseudo_path_max());
 		if (!pseudo_pwd_lck_name) {
@@ -415,21 +488,24 @@ pseudo_pwd_lck_open(void) {
 			return -1;
 		}
 	}
-	pseudo_pwd_lck_close();
+	pseudo_antimagic();
 	pseudo_pwd_lck_fd = PSEUDO_ETC_FILE(".pwd.lock",
 					pseudo_pwd_lck_name, O_RDWR | O_CREAT);
+	pseudo_magic();
 	return pseudo_pwd_lck_fd;
 }
 
 int
 pseudo_pwd_lck_close(void) {
 	if (pseudo_pwd_lck_fd != -1) {
+		pseudo_antimagic();
 		close(pseudo_pwd_lck_fd);
 		if (pseudo_pwd_lck_name) {
 			unlink(pseudo_pwd_lck_name);
 			free(pseudo_pwd_lck_name);
 			pseudo_pwd_lck_name = 0;
 		}
+		pseudo_magic();
 		pseudo_pwd_lck_fd = -1;
 		return 0;
 	} else {
